@@ -1,12 +1,5 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
-
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
 use std::io::Read;
 use std::path::{Path, PathBuf};
@@ -476,6 +469,7 @@ impl BuildDiagnostics {
     }
 
     #[cfg(feature = "display-diagnostics")]
+    #[must_use]
     pub fn check_and_exit_on_error(self) -> Self {
         if self.has_error() {
             self.print();
