@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 use crate::diagnostics::{BuildDiagnostics, SourceLocation, Spanned};
 use crate::langtype::{BuiltinElement, EnumerationValue, Type};
 use crate::layout::Orientation;
@@ -20,7 +14,7 @@ use std::rc::{Rc, Weak};
 // FIXME remove the pub
 pub use crate::namedreference::NamedReference;
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Copy)]
 /// A function built into the run-time
 pub enum BuiltinFunction {
     GetWindowScaleFactor,
@@ -37,6 +31,8 @@ pub enum BuiltinFunction {
     ACos,
     ASin,
     ATan,
+    Log,
+    Pow,
     SetFocusItem,
     ShowPopupWindow,
     /// the "42".to_float()
@@ -89,6 +85,10 @@ impl BuiltinFunction {
             BuiltinFunction::ACos | BuiltinFunction::ASin | BuiltinFunction::ATan => {
                 Type::Function { return_type: Box::new(Type::Angle), args: vec![Type::Float32] }
             }
+            BuiltinFunction::Log | BuiltinFunction::Pow => Type::Function {
+                return_type: Box::new(Type::Float32),
+                args: vec![Type::Float32, Type::Float32],
+            },
             BuiltinFunction::SetFocusItem => Type::Function {
                 return_type: Box::new(Type::Void),
                 args: vec![Type::ElementReference],
@@ -117,7 +117,7 @@ impl BuiltinFunction {
             },
             BuiltinFunction::ImageSize => Type::Function {
                 return_type: Box::new(Type::Struct {
-                    fields: std::array::IntoIter::new([
+                    fields: IntoIterator::into_iter([
                         ("width".to_string(), Type::Int32),
                         ("height".to_string(), Type::Int32),
                     ])
@@ -160,6 +160,8 @@ impl BuiltinFunction {
             | BuiltinFunction::Tan
             | BuiltinFunction::ACos
             | BuiltinFunction::ASin
+            | BuiltinFunction::Log
+            | BuiltinFunction::Pow
             | BuiltinFunction::ATan => true,
             BuiltinFunction::SetFocusItem => false,
             BuiltinFunction::ShowPopupWindow => false,
@@ -372,6 +374,13 @@ pub enum Expression {
         name: String,
     },
 
+    /// Access to a index within an array.
+    ArrayIndex {
+        /// This expression should have [`Type::Array`] type
+        array: Box<Expression>,
+        index: Box<Expression>,
+    },
+
     /// Cast an expression to the given type
     Cast {
         from: Box<Expression>,
@@ -429,9 +438,7 @@ pub enum Expression {
         values: HashMap<String, Expression>,
     },
 
-    PathElements {
-        elements: Path,
-    },
+    PathData(Path),
 
     EasingCurve(EasingCurve),
 
@@ -506,6 +513,10 @@ impl Expression {
                 Type::Component(c) => c.root_element.borrow().lookup_property(name).property_type,
                 _ => Type::Invalid,
             },
+            Expression::ArrayIndex { array, .. } => match array.ty() {
+                Type::Array(ty) => (*ty).clone(),
+                _ => Type::Invalid,
+            },
             Expression::Cast { to, .. } => to.clone(),
             Expression::CodeBlock(sub) => sub.last().map_or(Type::Void, |e| e.ty()),
             Expression::FunctionCall { function, .. } => match function.ty() {
@@ -583,7 +594,7 @@ impl Expression {
             Expression::UnaryOp { sub, .. } => sub.ty(),
             Expression::Array { element_ty, .. } => Type::Array(Box::new(element_ty.clone())),
             Expression::Struct { ty, .. } => ty.clone(),
-            Expression::PathElements { .. } => Type::PathElements,
+            Expression::PathData { .. } => Type::PathData,
             Expression::StoreLocalVariable { .. } => Type::Void,
             Expression::ReadLocalVariable { ty, .. } => ty.clone(),
             Expression::EasingCurve(_) => Type::Easing,
@@ -616,6 +627,10 @@ impl Expression {
             Expression::BuiltinMacroReference { .. } => {}
             Expression::ElementReference(_) => {}
             Expression::StructFieldAccess { base, .. } => visitor(&**base),
+            Expression::ArrayIndex { array, index } => {
+                visitor(&**array);
+                visitor(&**index);
+            }
             Expression::RepeaterIndexReference { .. } => {}
             Expression::RepeaterModelReference { .. } => {}
             Expression::Cast { from, .. } => visitor(&**from),
@@ -651,13 +666,16 @@ impl Expression {
                     visitor(x);
                 }
             }
-            Expression::PathElements { elements } => {
-                if let Path::Elements(elements) = elements {
+            Expression::PathData(data) => match data {
+                Path::Elements(elements) => {
                     for element in elements {
                         element.bindings.values().for_each(|binding| visitor(&binding.borrow()))
                     }
                 }
-            }
+                Path::Events(events, coordinates) => {
+                    events.iter().chain(coordinates.iter()).for_each(visitor);
+                }
+            },
             Expression::StoreLocalVariable { value, .. } => visitor(&**value),
             Expression::ReadLocalVariable { .. } => {}
             Expression::EasingCurve(_) => {}
@@ -698,6 +716,10 @@ impl Expression {
             Expression::BuiltinMacroReference { .. } => {}
             Expression::ElementReference(_) => {}
             Expression::StructFieldAccess { base, .. } => visitor(&mut **base),
+            Expression::ArrayIndex { array, index } => {
+                visitor(&mut **array);
+                visitor(&mut **index);
+            }
             Expression::RepeaterIndexReference { .. } => {}
             Expression::RepeaterModelReference { .. } => {}
             Expression::Cast { from, .. } => visitor(&mut **from),
@@ -733,8 +755,8 @@ impl Expression {
                     visitor(x);
                 }
             }
-            Expression::PathElements { elements } => {
-                if let Path::Elements(elements) = elements {
+            Expression::PathData(data) => match data {
+                Path::Elements(elements) => {
                     for element in elements {
                         element
                             .bindings
@@ -742,7 +764,10 @@ impl Expression {
                             .for_each(|binding| visitor(&mut binding.borrow_mut()))
                     }
                 }
-            }
+                Path::Events(events, coordinates) => {
+                    events.iter_mut().chain(coordinates.iter_mut()).for_each(visitor);
+                }
+            },
             Expression::StoreLocalVariable { value, .. } => visitor(&mut **value),
             Expression::ReadLocalVariable { .. } => {}
             Expression::EasingCurve(_) => {}
@@ -794,6 +819,7 @@ impl Expression {
             Expression::FunctionParameterReference { .. } => false,
             Expression::BuiltinMacroReference { .. } => true,
             Expression::StructFieldAccess { base, .. } => base.is_constant(),
+            Expression::ArrayIndex { array, index } => array.is_constant() && index.is_constant(),
             Expression::Cast { from, .. } => from.is_constant(),
             Expression::CodeBlock(sub) => sub.len() == 1 && sub.first().unwrap().is_constant(),
             Expression::FunctionCall { function, arguments, .. } => {
@@ -809,8 +835,8 @@ impl Expression {
             Expression::UnaryOp { sub, .. } => sub.is_constant(),
             Expression::Array { values, .. } => values.iter().all(Expression::is_constant),
             Expression::Struct { values, .. } => values.iter().all(|(_, v)| v.is_constant()),
-            Expression::PathElements { elements } => {
-                if let Path::Elements(elements) = elements {
+            Expression::PathData(data) => {
+                if let Path::Elements(elements) = data {
                     elements
                         .iter()
                         .all(|element| element.bindings.values().all(|v| v.borrow().is_constant()))
@@ -837,6 +863,7 @@ impl Expression {
     }
 
     /// Create a conversion node if needed, or throw an error if the type is not matching
+    #[must_use]
     pub fn maybe_convert_to(
         self,
         target_type: Type,
@@ -1021,7 +1048,7 @@ impl Expression {
             },
             Type::Bool => Expression::BoolLiteral(false),
             Type::Model => Expression::Invalid,
-            Type::PathElements => Expression::PathElements { elements: Path::Elements(vec![]) },
+            Type::PathData => Expression::PathData(Path::Elements(vec![])),
             Type::Array(element_ty) => {
                 Expression::Array { element_ty: (**element_ty).clone(), values: vec![] }
             }
@@ -1054,6 +1081,7 @@ impl Expression {
             }
             Expression::StructFieldAccess { base, .. } => base.try_set_rw(),
             Expression::RepeaterModelReference { .. } => true,
+            Expression::ArrayIndex { .. } => true,
             _ => false,
         }
     }
@@ -1172,15 +1200,16 @@ pub struct BindingAnalysis {
 
     /// true if the binding is a constant value that can be set without creating a binding at runtime
     pub is_const: bool,
-}
 
-pub type PathEvent = lyon_path::Event<lyon_path::math::Point, lyon_path::math::Point>;
-pub type PathEvents = Vec<PathEvent>;
+    /// true if this binding does not depends on the value of property that are set externally.
+    /// When true, this binding cannot be part of a binding loop involving external components
+    pub no_external_dependencies: bool,
+}
 
 #[derive(Debug, Clone)]
 pub enum Path {
     Elements(Vec<PathElement>),
-    Events(PathEvents),
+    Events(Vec<Expression>, Vec<Expression>),
 }
 
 #[derive(Debug, Clone)]
@@ -1248,6 +1277,12 @@ pub fn pretty_print(f: &mut dyn std::fmt::Write, expression: &Expression) -> std
             pretty_print(f, base)?;
             write!(f, ".{}", name)
         }
+        Expression::ArrayIndex { array, index } => {
+            pretty_print(f, array)?;
+            write!(f, "[")?;
+            pretty_print(f, index)?;
+            write!(f, "]")
+        }
         Expression::Cast { from, to } => {
             write!(f, "(")?;
             pretty_print(f, from)?;
@@ -1316,7 +1351,7 @@ pub fn pretty_print(f: &mut dyn std::fmt::Write, expression: &Expression) -> std
             }
             write!(f, " }}")
         }
-        Expression::PathElements { elements } => write!(f, "{:?}", elements),
+        Expression::PathData(data) => write!(f, "{:?}", data),
         Expression::EasingCurve(e) => write!(f, "{:?}", e),
         Expression::LinearGradient { angle, stops } => {
             write!(f, "@linear-gradient(")?;
