@@ -1,22 +1,15 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 /*!
 The module responsible for the code generation.
 
 There is one sub module for every language
 */
 
-use std::collections::{HashSet, VecDeque};
+use std::collections::{BTreeSet, HashSet, VecDeque};
 use std::rc::{Rc, Weak};
 
-use crate::diagnostics::BuildDiagnostics;
 use crate::expression_tree::{BindingExpression, Expression};
 use crate::langtype::Type;
 use crate::namedreference::NamedReference;
@@ -35,6 +28,7 @@ pub enum OutputFormat {
     #[cfg(feature = "rust")]
     Rust,
     Interpreter,
+    Llr,
 }
 
 impl OutputFormat {
@@ -57,6 +51,7 @@ impl std::str::FromStr for OutputFormat {
             "cpp" => Ok(Self::Cpp),
             #[cfg(feature = "rust")]
             "rust" => Ok(Self::Rust),
+            "llr" => Ok(Self::Llr),
             _ => Err(format!("Unknown outpout format {}", s)),
         }
     }
@@ -66,7 +61,6 @@ pub fn generate(
     format: OutputFormat,
     destination: &mut impl std::io::Write,
     doc: &Document,
-    diag: &mut BuildDiagnostics,
 ) -> std::io::Result<()> {
     #![allow(unused_variables)]
     #![allow(unreachable_code)]
@@ -79,15 +73,13 @@ pub fn generate(
     match format {
         #[cfg(feature = "cpp")]
         OutputFormat::Cpp => {
-            if let Some(output) = cpp::generate(doc, diag) {
-                write!(destination, "{}", output)?;
-            }
+            let output = cpp::generate(doc);
+            write!(destination, "{}", output)?;
         }
         #[cfg(feature = "rust")]
         OutputFormat::Rust => {
-            if let Some(output) = rust::generate(doc, diag) {
-                write!(destination, "{}", output)?;
-            }
+            let output = rust::generate(doc);
+            write!(destination, "{}", output)?;
         }
         OutputFormat::Interpreter => {
             return Err(std::io::Error::new(
@@ -95,6 +87,13 @@ pub fn generate(
                 "Unsupported output format: The interpreter is not a valid output format yet.",
             )); // Perhaps byte code in the future?
         }
+        OutputFormat::Llr => {
+            writeln!(
+                destination,
+                "{:#?}",
+                crate::llr::lower_to_item_tree::lower_to_item_tree(&doc.root_component)
+            )?;
+        }
     }
     Ok(())
 }
@@ -147,7 +146,7 @@ pub fn build_item_tree<T: ItemTreeBuilder>(
     if let Some(sub_component) = root_component.root_element.borrow().sub_component() {
         assert!(root_component.root_element.borrow().children.is_empty());
         let sub_compo_state =
-            builder.enter_component(&root_component.root_element, &sub_component, 1, initial_state);
+            builder.enter_component(&root_component.root_element, sub_component, 1, initial_state);
         builder.enter_component_children(
             &root_component.root_element,
             0,
@@ -162,7 +161,7 @@ pub fn build_item_tree<T: ItemTreeBuilder>(
         visit_children(
             initial_state,
             &root_component.root_element.borrow().children,
-            &root_component,
+            root_component,
             &root_component.root_element,
             0,
             0,
@@ -177,12 +176,10 @@ pub fn build_item_tree<T: ItemTreeBuilder>(
     // sub-component children, needed to allocate the correct amount of
     // index spaces for sub-components.
     fn item_sub_tree_size(e: &ElementRc) -> usize {
-        let e = if let Some(sub_component) = e.borrow().sub_component() {
-            sub_component.root_element.clone()
-        } else {
-            e.clone()
-        };
         let mut count = e.borrow().children.len();
+        if let Some(sub_component) = e.borrow().sub_component() {
+            count += item_sub_tree_size(&sub_component.root_element);
+        }
         for i in &e.borrow().children {
             count += item_sub_tree_size(i);
         }
@@ -230,7 +227,7 @@ pub fn build_item_tree<T: ItemTreeBuilder>(
                 visit_children(
                     &sub_component_state,
                     &nested_subcomponent.root_element.borrow().children,
-                    &nested_subcomponent,
+                    nested_subcomponent,
                     &nested_subcomponent.root_element,
                     parent_index,
                     relative_parent_index,
@@ -250,7 +247,7 @@ pub fn build_item_tree<T: ItemTreeBuilder>(
         for child in children.iter() {
             if let Some(sub_component) = child.borrow().sub_component() {
                 let sub_component_state =
-                    builder.enter_component(child, &sub_component, offset, state);
+                    builder.enter_component(child, sub_component, offset, state);
                 visit_item(
                     &sub_component_state,
                     &sub_component.root_element,
@@ -321,7 +318,6 @@ pub fn build_item_tree<T: ItemTreeBuilder>(
         if item.borrow().repeated.is_some() {
             builder.push_repeated_item(item, *repeater_count, parent_index, component_state);
             *repeater_count += 1;
-            return;
         } else {
             let mut item = item.clone();
             let mut component_state = component_state.clone();
@@ -329,7 +325,7 @@ pub fn build_item_tree<T: ItemTreeBuilder>(
                 let base = item.borrow().sub_component().map(|c| {
                     (
                         c.root_element.clone(),
-                        builder.enter_component(&item, &c, children_offset, &component_state),
+                        builder.enter_component(&item, c, children_offset, &component_state),
                     )
                 });
                 base
@@ -400,3 +396,48 @@ pub fn handle_property_bindings_init(
         }
     });
 }
+
+/// Call the given function for each constant property in the Component so one can set
+/// `set_constant` on it.
+pub fn for_each_const_properties(component: &Rc<Component>, mut f: impl FnMut(&ElementRc, &str)) {
+    crate::object_tree::recurse_elem(&component.root_element, &(), &mut |elem: &ElementRc, ()| {
+        if elem.borrow().repeated.is_some() {
+            return;
+        }
+        let mut e = elem.clone();
+        let mut all_prop = BTreeSet::new();
+        loop {
+            all_prop.extend(
+                e.borrow()
+                    .property_declarations
+                    .iter()
+                    .filter(|(_, x)| x.property_type.is_property_type())
+                    .map(|(k, _)| k.clone()),
+            );
+            match &e.clone().borrow().base_type {
+                Type::Component(c) => {
+                    e = c.root_element.clone();
+                }
+                Type::Native(n) => {
+                    all_prop.extend(
+                        n.properties
+                            .iter()
+                            .filter(|(k, x)| {
+                                x.ty.is_property_type()
+                                    && !k.starts_with("viewport-")
+                                    && k.as_str() != "commands"
+                            })
+                            .map(|(k, _)| k.clone()),
+                    );
+                    break;
+                }
+                _ => break,
+            }
+        }
+        for c in all_prop {
+            if NamedReference::new(elem, &c).is_constant() {
+                f(elem, &c);
+            }
+        }
+    });
+}
