@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 //! Datastructures used to represent layouts in the compiler
 
 use crate::diagnostics::BuildDiagnostics;
@@ -393,7 +387,7 @@ fn find_binding<R>(
 
 /// Return a named reference to a property if a binding is set on that property
 fn binding_reference(element: &ElementRc, name: &str) -> Option<NamedReference> {
-    find_binding(element, name, |_, _, _| NamedReference::new(&element, name))
+    find_binding(element, name, |_, _, _| NamedReference::new(element, name))
 }
 
 fn init_fake_property(
