@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 /*!
 This module contains the [`NamedReference`] and its helper
 */
@@ -104,7 +98,6 @@ impl NamedReference {
                 return true;
             }
             match &e.base_type {
-                Type::Native(_) => return false, // after resolving we don't know anymore if the property can be changed natively
                 Type::Component(c) => {
                     let next = c.root_element.clone();
                     drop(e);
@@ -114,6 +107,9 @@ impl NamedReference {
                 Type::Builtin(b) => {
                     return b.properties.get(self.name()).map_or(true, |pi| !pi.is_native_output)
                 }
+                Type::Native(n) => {
+                    return n.properties.get(self.name()).map_or(true, |pi| !pi.is_native_output)
+                }
                 _ => return true,
             }
         }
@@ -196,7 +192,7 @@ impl NamedReferenceContainer {
     }
 }
 
-/// Mark that a given property is `is_set_derived` in all bases
+/// Mark that a given property is `is_set_externally` in all bases
 pub(crate) fn mark_property_set_derived_in_base(mut element: ElementRc, name: &str) {
     loop {
         let next = if let Type::Component(c) = &element.borrow().base_type {
