@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 /*!
  This module contains the intermediate representation of the code in the form of an object tree
 */
@@ -138,7 +132,21 @@ impl Document {
                     || import.file.ends_with(".ttf")
                     || import.file.ends_with(".otf")
                 {
-                    Some(import.file)
+                    // Assume remote urls are valid, we need to load them at run-time (which we currently don't). For
+                    // local paths we should try to verify the existence and let the developer know ASAP.
+                    if import.file.starts_with("http://")
+                        || import.file.starts_with("https://")
+                        || crate::fileaccess::load_file(std::path::Path::new(&import.file))
+                            .is_some()
+                    {
+                        Some(import.file)
+                    } else {
+                        diag.push_error(
+                            format!("File \"{}\" not found", import.file),
+                            &import.import_token,
+                        );
+                        None
+                    }
                 } else {
                     diag.push_error(
                         format!("Unsupported foreign import \"{}\"", import.file),
@@ -288,7 +296,7 @@ impl Component {
         self.exported_global_names
             .borrow()
             .iter()
-            .filter(|name| name.as_str() != &self.root_element.borrow().id)
+            .filter(|name| name.as_str() != self.root_element.borrow().id)
             .map(|name| name.original_name())
             .collect()
     }
@@ -453,6 +461,8 @@ pub struct Element {
     /// This is the component-local index of this item in the item tree array.
     /// It is generated after the last pass and before the generators run.
     pub item_index: once_cell::unsync::OnceCell<usize>,
+    /// the index of the first children in the tree, set with item_index
+    pub item_index_of_first_children: once_cell::unsync::OnceCell<usize>,
 
     /// The AST node, if available
     pub node: Option<syntax_nodes::Element>,
@@ -1238,10 +1248,7 @@ pub fn type_from_node(
         let prop_type = tr.lookup_qualified(&qualified_type.members);
 
         if prop_type == Type::Invalid {
-            diag.push_error(
-                format!("Unknown type '{}'", qualified_type.to_string()),
-                &qualified_type_node,
-            );
+            diag.push_error(format!("Unknown type '{}'", qualified_type), &qualified_type_node);
         }
         prop_type
     } else if let Some(object_node) = node.ObjectType() {
@@ -1488,7 +1495,7 @@ pub fn recurse_elem_including_sub_components_no_borrow<State>(
         .borrow()
         .globals
         .iter()
-        .for_each(|p| recurse_elem_including_sub_components_no_borrow(&p, state, vis));
+        .for_each(|p| recurse_elem_including_sub_components_no_borrow(p, state, vis));
 }
 
 /// This visit the binding attached to this element, but does not recurse in children elements
@@ -1608,7 +1615,7 @@ pub fn visit_all_named_references_in_element(
     elem.borrow_mut().layout_info_prop = layout_info_prop;
 
     // visit two way bindings
-    for (_, expr) in &elem.borrow().bindings {
+    for expr in elem.borrow().bindings.values() {
         for nr in &mut expr.borrow_mut().two_way_bindings {
             vis(nr);
         }
@@ -1722,7 +1729,7 @@ impl Exports {
                         parser::identifier_text(&ident).map(|text| (text, ident.clone().into()))
                     })
                     .unwrap_or_else(|| {
-                        (internal_name.clone(), export_specifier.ExportIdentifier().clone().into())
+                        (internal_name.clone(), export_specifier.ExportIdentifier().into())
                     });
 
                 NamedExport {
@@ -1862,6 +1869,26 @@ pub fn inject_element_as_repeated_element(repeated_element: &ElementRc, new_root
     debug_assert_eq!(Rc::strong_count(&component), 2);
     let old_root = &component.root_element;
 
+    // The values for properties that affect the geometry may be supplied in two different ways:
+    //
+    //   * When coming from the outside, for example by the repeater being inside a layout, we need
+    //     the values to apply to the new root element and the old root just needs to follow.
+    //   * When coming from the inside, for example when the repeater just creates rectangles that
+    //     calculate their own position, we need to move those bindings as well to the new root.
+    //
+    //  Finally, the default geometry pass lowering following the passes that call this function
+    //  will apply a binding to the width and height of the inner to follow the size of the parent
+    //  (the new root).
+    {
+        let mut old_root = old_root.borrow_mut();
+        for (binding_to_move, _) in crate::typeregister::RESERVED_GEOMETRY_PROPERTIES.iter() {
+            let binding_to_move = binding_to_move.to_string();
+            if let Some(binding) = old_root.bindings.remove(&binding_to_move) {
+                new_root.borrow_mut().bindings.insert(binding_to_move, binding);
+            }
+        }
+    }
+
     // Any elements with a weak reference to the repeater's component will need fixing later.
     let mut elements_with_enclosing_component_reference = Vec::new();
     recurse_elem(old_root, &(), &mut |element: &ElementRc, _| {
