@@ -1,12 +1,5 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
-
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
 mod apply_default_properties_from_style;
 mod binding_analysis;
@@ -46,12 +39,14 @@ mod unique_id;
 mod visible;
 mod z_order;
 
+use std::rc::Rc;
+
 use crate::langtype::Type;
 
 pub async fn run_passes(
     doc: &crate::object_tree::Document,
     diag: &mut crate::diagnostics::BuildDiagnostics,
-    mut type_loader: &mut crate::typeloader::TypeLoader<'_>,
+    type_loader: &mut crate::typeloader::TypeLoader<'_>,
     compiler_config: &crate::CompilerConfiguration,
 ) {
     if matches!(doc.root_component.root_element.borrow().base_type, Type::Invalid | Type::Void) {
@@ -59,6 +54,19 @@ pub async fn run_passes(
         return;
     }
 
+    let style_metrics = {
+        // Ignore import errors
+        let mut build_diags_to_ignore = crate::diagnostics::BuildDiagnostics::default();
+        let style_metrics = type_loader
+            .import_type("sixtyfps_widgets.60", "StyleMetrics", &mut build_diags_to_ignore)
+            .await;
+        if let Some(Type::Component(c)) = style_metrics {
+            c
+        } else {
+            panic!("can't load style metrics")
+        }
+    };
+
     let global_type_registry = type_loader.global_type_registry.clone();
     let root_component = &doc.root_component;
     infer_aliases_types::resolve_aliases(doc, diag);
@@ -72,7 +80,13 @@ pub async fn run_passes(
         .chain(std::iter::once(root_component))
     {
         compile_paths::compile_paths(component, &doc.local_registry, diag);
-        lower_tabwidget::lower_tabwidget(component, &mut type_loader, diag).await;
+        lower_tabwidget::lower_tabwidget(component, type_loader, diag).await;
+        apply_default_properties_from_style::apply_default_properties_from_style(
+            component,
+            &style_metrics,
+            diag,
+        );
+        lower_states::lower_states(component, &doc.local_registry, diag);
     }
 
     inlining::inline(doc, inlining::InlineSelection::InlineOnlyRequiredComponents);
@@ -84,39 +98,38 @@ pub async fn run_passes(
         .chain(std::iter::once(root_component))
     {
         focus_item::resolve_element_reference_in_set_focus_calls(component, diag);
-        focus_item::determine_initial_focus_item(component, diag);
+        if Rc::ptr_eq(component, root_component) {
+            focus_item::determine_initial_focus_item(component, diag);
+        }
         focus_item::erase_forward_focus_properties(component);
     }
 
-    ensure_window::ensure_window(root_component, &doc.local_registry);
+    ensure_window::ensure_window(root_component, &doc.local_registry, &style_metrics);
 
     for component in (root_component.used_types.borrow().sub_components.iter())
         .chain(std::iter::once(root_component))
     {
-        apply_default_properties_from_style::apply_default_properties_from_style(
-            component,
-            &mut type_loader,
-            diag,
-        )
-        .await;
         flickable::handle_flickable(component, &global_type_registry.borrow());
-        lower_states::lower_states(component, &doc.local_registry, diag);
         repeater_component::process_repeater_components(component);
         lower_popups::lower_popups(component, &doc.local_registry, diag);
-        lower_layout::lower_layouts(component, &mut type_loader, diag).await;
+        lower_layout::lower_layouts(component, type_loader, diag).await;
         z_order::reorder_by_z_order(component, diag);
-        lower_shadows::lower_shadow_properties(component, &doc.local_registry, diag);
-        clip::handle_clip(component, &global_type_registry.borrow(), diag);
         transform_and_opacity::handle_transform_and_opacity(
             component,
             &global_type_registry.borrow(),
             diag,
         );
+        lower_shadows::lower_shadow_properties(component, &doc.local_registry, diag);
+        clip::handle_clip(component, &global_type_registry.borrow(), diag);
+        // Apply the default geometry after any synthetic elements may have been injected
+        // as root in repeaters(opacity, box shadow, etc.).
+        // The geometry bindings are moved to the new injected roots, but the previous elements
+        // rely on the default geometry pass to size them correctly relative to the new root.
         default_geometry::default_geometry(component, diag);
         visible::handle_visible(component, &global_type_registry.borrow());
         materialize_fake_properties::materialize_fake_properties(component);
     }
-    collect_globals::collect_globals(&doc, diag);
+    collect_globals::collect_globals(doc, diag);
 
     if compiler_config.inline_all_elements {
         inlining::inline(doc, inlining::InlineSelection::InlineAllComponents);
@@ -133,7 +146,10 @@ pub async fn run_passes(
         optimize_useless_rectangles::optimize_useless_rectangles(component);
         move_declarations::move_declarations(component, diag);
         remove_aliases::remove_aliases(component, diag);
-        const_propagation::const_propagation(component);
+        if !diag.has_error() {
+            // binding loop causes panics in const_propagation
+            const_propagation::const_propagation(component);
+        }
     }
 
     for component in (root_component.used_types.borrow().sub_components.iter())
@@ -143,7 +159,7 @@ pub async fn run_passes(
         remove_unused_properties::remove_unused_properties(component);
     }
 
-    collect_structs::collect_structs(&doc);
+    collect_structs::collect_structs(doc);
 
     for component in (root_component.used_types.borrow().sub_components.iter())
         .chain(std::iter::once(root_component))
@@ -156,6 +172,8 @@ pub async fn run_passes(
         std::iter::once(&*doc).chain(type_loader.all_documents()),
         compiler_config.embed_resources,
     );
+    // collect globals once more: After optimizations we might have less globals
+    collect_globals::collect_globals(doc, diag);
     root_component.is_root_component.set(true);
 }
 
