@@ -1,12 +1,5 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
-
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
 //! Passes that fills the root component used_types.globals
 
@@ -20,6 +13,7 @@ use std::collections::HashSet;
 
 /// Fill the root_component's used_types.globals
 pub fn collect_globals(doc: &Document, _diag: &mut BuildDiagnostics) {
+    doc.root_component.used_types.borrow_mut().globals.clear();
     let mut set = HashSet::new();
     for (_, ty) in doc.exports() {
         if let Type::Component(c) = ty {
