@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 //! Inline each object_tree::Component within the main Component
 
 use crate::expression_tree::{BindingExpression, Expression, NamedReference};
@@ -91,11 +85,9 @@ fn inline_element(
         elem_mut.property_analysis.borrow_mut().entry(p.clone()).or_default().merge_with_base(a);
     }
 
-    // FIXME: states and transitions will be merged while inlining, this is not what we want
-    elem_mut.states.extend(inlined_component.root_element.borrow().states.iter().cloned());
-    elem_mut
-        .transitions
-        .extend(inlined_component.root_element.borrow().transitions.iter().cloned());
+    // states and transitions must be lowered before inlining
+    debug_assert!(inlined_component.root_element.borrow().states.is_empty());
+    debug_assert!(inlined_component.root_element.borrow().transitions.is_empty());
 
     // Map the old element to the new
     let mut mapping = HashMap::new();
@@ -134,14 +126,11 @@ fn inline_element(
 
     elem_mut.children = new_children;
 
-    match &mut elem_mut.base_type {
-        Type::Component(c) => {
-            if c.parent_element.upgrade().is_some() {
-                debug_assert!(Rc::ptr_eq(elem, &c.parent_element.upgrade().unwrap()));
-                *c = duplicate_sub_component(c, elem, &mut mapping);
-            }
+    if let Type::Component(c) = &mut elem_mut.base_type {
+        if c.parent_element.upgrade().is_some() {
+            debug_assert!(Rc::ptr_eq(elem, &c.parent_element.upgrade().unwrap()));
+            *c = duplicate_sub_component(c, elem, &mut mapping);
         }
-        _ => {}
     };
 
     root_component.optimized_elements.borrow_mut().extend(
@@ -226,17 +215,15 @@ fn duplicate_element_with_mapping(
         layout_info_prop: elem.layout_info_prop.clone(),
         named_references: Default::default(),
         item_index: Default::default(), // Not determined yet
+        item_index_of_first_children: Default::default(),
         is_flickable_viewport: elem.is_flickable_viewport,
     }));
     mapping.insert(element_key(element.clone()), new.clone());
-    match &mut new.borrow_mut().base_type {
-        Type::Component(c) => {
-            if c.parent_element.upgrade().is_some() {
-                debug_assert!(Rc::ptr_eq(element, &c.parent_element.upgrade().unwrap()));
-                *c = duplicate_sub_component(c, &new, mapping);
-            }
+    if let Type::Component(c) = &mut new.borrow_mut().base_type {
+        if c.parent_element.upgrade().is_some() {
+            debug_assert!(Rc::ptr_eq(element, &c.parent_element.upgrade().unwrap()));
+            *c = duplicate_sub_component(c, &new, mapping);
         }
-        _ => (),
     };
 
     new
@@ -287,13 +274,13 @@ fn duplicate_sub_component(
         .map(|p| duplicate_popup(p, mapping))
         .collect();
     for p in new_component.popup_windows.borrow_mut().iter_mut() {
-        fixup_reference(&mut p.x, &mapping);
-        fixup_reference(&mut p.y, &mapping);
+        fixup_reference(&mut p.x, mapping);
+        fixup_reference(&mut p.y, mapping);
     }
     new_component
         .root_constraints
         .borrow_mut()
-        .visit_named_references(&mut |nr| fixup_reference(nr, &mapping));
+        .visit_named_references(&mut |nr| fixup_reference(nr, mapping));
     new_component
 }
 
@@ -455,11 +442,9 @@ fn component_requires_inlining(component: &Rc<Component>) -> bool {
         if prop.starts_with("drop-shadow-") || prop == "opacity" {
             return true;
         }
-        if prop == "height" || prop == "width" {
-            if binding.expression.ty() == Type::Percent {
-                // percentage size in the root element might not make sense anyway.
-                return true;
-            }
+        if (prop == "height" || prop == "width") && binding.expression.ty() == Type::Percent {
+            // percentage size in the root element might not make sense anyway.
+            return true;
         }
         if binding.animation.is_some() {
             // If there is an animation, we currently inline so that if this property
