@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 //! This pass moves all declaration of properties or callback to the root
 
 use crate::diagnostics::BuildDiagnostics;
@@ -81,11 +75,11 @@ pub fn move_declarations(component: &Rc<Component>, diag: &mut BuildDiagnostics)
     component.optimized_elements.borrow().iter().for_each(|e| move_bindings_and_animations(e));
     recurse_elem(&component.root_element, &(), &mut |e, _| move_bindings_and_animations(e));
 
-    component.root_constraints.borrow_mut().visit_named_references(&mut |e| fixup_reference(e));
+    component.root_constraints.borrow_mut().visit_named_references(&mut fixup_reference);
     component.popup_windows.borrow_mut().iter_mut().for_each(|p| {
         fixup_reference(&mut p.x);
         fixup_reference(&mut p.y);
-        visit_all_named_references(&p.component, &mut |e| fixup_reference(e))
+        visit_all_named_references(&p.component, &mut fixup_reference)
     });
     for pd in decl.property_declarations.values_mut() {
         pd.is_alias.as_mut().map(fixup_reference);
