@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 //! This pass removes the property used in a two ways bindings
 
 use crate::diagnostics::BuildDiagnostics;
@@ -69,7 +63,8 @@ impl PropertySets {
 pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
     // collect all sets that are linked together
     let mut property_sets = PropertySets::default();
-    recurse_elem_including_sub_components(component, &(), &mut |e, _| {
+
+    let mut process_element = |e: &ElementRc| {
         'bindings: for (name, binding) in &e.borrow().bindings {
             for nr in &binding.borrow().two_way_bindings {
                 let other_e = nr.element();
@@ -80,7 +75,12 @@ pub fn remove_aliases(component: &Rc<Component>, diag: &mut BuildDiagnostics) {
                 property_sets.add_link(NamedReference::new(e, name), nr.clone());
             }
         }
-    });
+    };
+
+    recurse_elem_including_sub_components(component, &(), &mut |e, &()| process_element(e));
+    for g in &component.used_types.borrow().globals {
+        process_element(&g.root_element)
+    }
 
     // The key will be removed and replaced by the named reference
     let mut aliases_to_remove = Mapping::new();
