@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 /*!
 Make sure that the Repeated expression are just components without any children
  */
@@ -49,6 +43,7 @@ fn create_repeater_components(component: &Rc<Component>) {
                 layout_info_prop: elem.layout_info_prop.take(),
                 is_flickable_viewport: elem.is_flickable_viewport,
                 item_index: Default::default(), // Not determined yet
+                item_index_of_first_children: Default::default(),
             })),
             parent_element,
             ..Component::default()
@@ -71,6 +66,14 @@ fn create_repeater_components(component: &Rc<Component>) {
                     RefCell::new(Expression::PropertyReference(listview.listview_width).into()),
                 );
             }
+
+            comp.root_element
+                .borrow()
+                .property_analysis
+                .borrow_mut()
+                .entry("y".into())
+                .or_default()
+                .is_set_externally = true;
         }
 
         let weak = Rc::downgrade(&comp);
