@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 //! Passes that resolve the property binding expression.
 //!
 //! Before this pass, all the expression are of type Expression::Uncompiled,
@@ -17,7 +11,7 @@ LICENSE END */
 use crate::diagnostics::{BuildDiagnostics, Spanned};
 use crate::expression_tree::*;
 use crate::langtype::{PropertyLookupResult, Type};
-use crate::lookup::{LookupCtx, LookupObject};
+use crate::lookup::{LookupCtx, LookupObject, LookupResult};
 use crate::object_tree::*;
 use crate::parser::{identifier_text, syntax_nodes, NodeOrToken, SyntaxKind, SyntaxNode};
 use crate::typeregister::TypeRegister;
@@ -169,7 +163,7 @@ impl Expression {
                 return Expression::Invalid;
             }
         };
-        e.maybe_convert_to(ctx.property_type.clone(), &node, &mut ctx.diag)
+        e.maybe_convert_to(ctx.property_type.clone(), &node, ctx.diag)
     }
 
     fn from_codeblock_node(node: syntax_nodes::CodeBlock, ctx: &mut LookupCtx) -> Expression {
@@ -204,7 +198,7 @@ impl Expression {
 
         exit_points_and_return_types.into_iter().for_each(|(index, _)| {
             let mut expr = std::mem::replace(&mut statements_or_exprs[index], Expression::Invalid);
-            expr = expr.maybe_convert_to(common_return_type.clone(), &node, &mut ctx.diag);
+            expr = expr.maybe_convert_to(common_return_type.clone(), &node, ctx.diag);
             statements_or_exprs[index] = expr;
         });
 
@@ -220,7 +214,7 @@ impl Expression {
             Box::new(Self::from_expression_node(n, ctx).maybe_convert_to(
                 return_type,
                 &node,
-                &mut ctx.diag,
+                ctx.diag,
             ))
         }))
     }
@@ -234,7 +228,7 @@ impl Expression {
         Self::from_codeblock_node(node.CodeBlock(), ctx).maybe_convert_to(
             ctx.return_type().clone(),
             &node,
-            &mut ctx.diag,
+            ctx.diag,
         )
     }
 
@@ -251,8 +245,7 @@ impl Expression {
                             format!(
                                 "'{}' must be called. Did you forgot the '()'?",
                                 QualifiedTypeName::from_node(n.clone())
-                            )
-                            .into(),
+                            ),
                             &n,
                         )
                     }
@@ -294,6 +287,8 @@ impl Expression {
             .or_else(|| {
                 node.FunctionCallExpression().map(|n| Self::from_function_call_node(n, ctx))
             })
+            .or_else(|| node.MemberAccess().map(|n| Self::from_member_access_node(n, ctx)))
+            .or_else(|| node.IndexExpression().map(|n| Self::from_index_expression_node(n, ctx)))
             .or_else(|| node.SelfAssignment().map(|n| Self::from_self_assignment_node(n, ctx)))
             .or_else(|| node.BinaryExpression().map(|n| Self::from_binary_expression_node(n, ctx)))
             .or_else(|| {
@@ -373,7 +368,7 @@ impl Expression {
             Box::new(Expression::from_expression_node(angle_expr.clone(), ctx).maybe_convert_to(
                 Type::Angle,
                 &angle_expr,
-                &mut ctx.diag,
+                ctx.diag,
             ));
 
         let mut stops = vec![];
@@ -411,15 +406,14 @@ impl Expression {
                 };
                 match std::mem::replace(&mut current_stop, Stop::Finished) {
                     Stop::Empty => {
-                        current_stop =
-                            Stop::Color(e.maybe_convert_to(Type::Color, &n, &mut ctx.diag))
+                        current_stop = Stop::Color(e.maybe_convert_to(Type::Color, &n, ctx.diag))
                     }
                     Stop::Finished => {
                         ctx.diag.push_error("Expected comma".into(), &n);
                         break;
                     }
                     Stop::Color(col) => {
-                        stops.push((col, e.maybe_convert_to(Type::Float32, &n, &mut ctx.diag)))
+                        stops.push((col, e.maybe_convert_to(Type::Float32, &n, ctx.diag)))
                     }
                 }
             }
@@ -521,49 +515,73 @@ impl Expression {
             Some(x) => x,
         };
 
-        if let Some(depr) = result.deprecated {
-            ctx.diag.push_property_deprecation_warning(&first_str, &depr, &first);
+        if let Some(depr) = result.deprecated() {
+            ctx.diag.push_property_deprecation_warning(&first_str, depr, &first);
         }
 
-        match result.expression {
-            Expression::ElementReference(e) => {
+        match result {
+            LookupResult::Expression { expression: Expression::ElementReference(e), .. } => {
                 continue_lookup_within_element(&e.upgrade().unwrap(), &mut it, node, ctx)
             }
-            r @ Expression::CallbackReference(..) => {
+            LookupResult::Expression {
+                expression: r @ Expression::CallbackReference(..), ..
+            } => {
                 if let Some(x) = it.next() {
                     ctx.diag.push_error("Cannot access fields of callback".into(), &x)
                 }
                 r
             }
-            Expression::EnumerationValue(ev) => {
-                // Special meaning for the type itself
-                if ev.value == usize::MAX {
-                    if let Some(next_identifier) = it.next() {
-                        match ev.enumeration.lookup(
-                            ctx,
-                            &crate::parser::normalize_identifier(next_identifier.text()),
-                        ) {
-                            None => {
-                                ctx.diag.push_error(
-                                    format!(
-                                        "'{}' is not a member of the enum {}",
-                                        next_identifier.text(),
-                                        ev.enumeration.name
-                                    ),
-                                    &next_identifier,
-                                );
-                                return Expression::Invalid;
-                            }
-                            Some(r) => return maybe_lookup_object(r.expression, it, ctx),
+            LookupResult::Enumeration(enumeration) => {
+                if let Some(next_identifier) = it.next() {
+                    match enumeration
+                        .lookup(ctx, &crate::parser::normalize_identifier(next_identifier.text()))
+                    {
+                        Some(LookupResult::Expression { expression, .. }) => {
+                            maybe_lookup_object(expression, it, ctx)
+                        }
+                        _ => {
+                            ctx.diag.push_error(
+                                format!(
+                                    "'{}' is not a member of the enum {}",
+                                    next_identifier.text(),
+                                    enumeration.name
+                                ),
+                                &next_identifier,
+                            );
+                            Expression::Invalid
                         }
-                    } else {
-                        ctx.diag.push_error("Cannot take reference to an enum".to_string(), &node);
-                        return Expression::Invalid;
                     }
+                } else {
+                    ctx.diag.push_error("Cannot take reference to an enum".to_string(), &node);
+                    Expression::Invalid
+                }
+            }
+            LookupResult::Expression { expression, .. } => maybe_lookup_object(expression, it, ctx),
+            LookupResult::Namespace(_) => {
+                if let Some(next_identifier) = it.next() {
+                    match result
+                        .lookup(ctx, &crate::parser::normalize_identifier(next_identifier.text()))
+                    {
+                        Some(LookupResult::Expression { expression, .. }) => {
+                            maybe_lookup_object(expression, it, ctx)
+                        }
+                        _ => {
+                            ctx.diag.push_error(
+                                format!(
+                                    "'{}' is not a member of the namespace {}",
+                                    next_identifier.text(),
+                                    first_str
+                                ),
+                                &next_identifier,
+                            );
+                            Expression::Invalid
+                        }
+                    }
+                } else {
+                    ctx.diag.push_error("Cannot take reference to a namespace".to_string(), &node);
+                    Expression::Invalid
                 }
-                maybe_lookup_object(Expression::EnumerationValue(ev), it, ctx)
             }
-            other => maybe_lookup_object(other, it, ctx),
         }
     }
 
@@ -575,22 +593,27 @@ impl Expression {
 
         let mut sub_expr = node.Expression();
 
-        let function = sub_expr
-            .next()
-            .and_then(|n| {
-                n.QualifiedName().or_else(|| {
+        let function = sub_expr.next().map_or(Self::Invalid, |n| {
+            // Treat the QualifiedName separately so we can catch the uses of uncalled signal
+            n.QualifiedName()
+                .or_else(|| {
                     n.Expression().and_then(|mut e| {
                         while let Some(e2) = e.Expression() {
                             e = e2;
                         }
                         e.QualifiedName().map(|q| {
-                            ctx.diag.push_warning("Parentheses around callable are deprecated. Remove the parentheses".into(), &n);
+                            ctx.diag.push_warning(
+                            "Parentheses around callable are deprecated. Remove the parentheses"
+                                .into(),
+                            &n,
+                        );
                             q
                         })
                     })
                 })
-            })
-            .map_or(Expression::Invalid, |n| Self::from_qualified_name_node(n.clone(), ctx));
+                .map(|qn| Self::from_qualified_name_node(qn, ctx))
+                .unwrap_or_else(|| Self::from_expression_node(n, ctx))
+        });
 
         let sub_expr = sub_expr.map(|n| {
             (Self::from_expression_node(n.clone(), ctx), Some(NodeOrToken::from((*n).clone())))
@@ -599,12 +622,7 @@ impl Expression {
         let function = match function {
             Expression::BuiltinMacroReference(mac, n) => {
                 arguments.extend(sub_expr);
-                return crate::builtin_macros::lower_macro(
-                    mac,
-                    n,
-                    arguments.into_iter(),
-                    &mut ctx.diag,
-                );
+                return crate::builtin_macros::lower_macro(mac, n, arguments.into_iter(), ctx.diag);
             }
             Expression::MemberFunction { base, base_node, member } => {
                 arguments.push((*base, base_node));
@@ -630,7 +648,7 @@ impl Expression {
                     arguments
                         .into_iter()
                         .zip(args.iter())
-                        .map(|((e, node), ty)| e.maybe_convert_to(ty.clone(), &node, &mut ctx.diag))
+                        .map(|((e, node), ty)| e.maybe_convert_to(ty.clone(), &node, ctx.diag))
                         .collect()
                 }
             }
@@ -647,6 +665,14 @@ impl Expression {
         }
     }
 
+    fn from_member_access_node(
+        node: syntax_nodes::MemberAccess,
+        ctx: &mut LookupCtx,
+    ) -> Expression {
+        let base = Self::from_expression_node(node.Expression(), ctx);
+        maybe_lookup_object(base, node.child_token(SyntaxKind::Identifier).into_iter(), ctx)
+    }
+
     fn from_self_assignment_node(
         node: syntax_nodes::SelfAssignment,
         ctx: &mut LookupCtx,
@@ -688,7 +714,7 @@ impl Expression {
         let rhs = Self::from_expression_node(rhs_n.clone(), ctx);
         Expression::SelfAssignment {
             lhs: Box::new(lhs),
-            rhs: Box::new(rhs.maybe_convert_to(expected_ty, &rhs_n, &mut ctx.diag)),
+            rhs: Box::new(rhs.maybe_convert_to(expected_ty, &rhs_n, ctx.diag)),
             op,
         }
     }
@@ -755,7 +781,7 @@ impl Expression {
                                 rhs: Box::new(rhs.maybe_convert_to(
                                     Type::Float32,
                                     &rhs_n,
-                                    &mut ctx.diag,
+                                    ctx.diag,
                                 )),
                                 op,
                             }
@@ -765,7 +791,7 @@ impl Expression {
                                 lhs: Box::new(lhs.maybe_convert_to(
                                     Type::Float32,
                                     &lhs_n,
-                                    &mut ctx.diag,
+                                    ctx.diag,
                                 )),
                                 rhs: Box::new(rhs),
                                 op,
@@ -779,8 +805,8 @@ impl Expression {
             }
         };
         Expression::BinaryExpression {
-            lhs: Box::new(lhs.maybe_convert_to(expected_ty.clone(), &lhs_n, &mut ctx.diag)),
-            rhs: Box::new(rhs.maybe_convert_to(expected_ty, &rhs_n, &mut ctx.diag)),
+            lhs: Box::new(lhs.maybe_convert_to(expected_ty.clone(), &lhs_n, ctx.diag)),
+            rhs: Box::new(rhs.maybe_convert_to(expected_ty, &rhs_n, ctx.diag)),
             op,
         }
     }
@@ -811,15 +837,15 @@ impl Expression {
         let condition = Self::from_expression_node(condition_n.clone(), ctx).maybe_convert_to(
             Type::Bool,
             &condition_n,
-            &mut ctx.diag,
+            ctx.diag,
         );
         let true_expr = Self::from_expression_node(true_expr_n.clone(), ctx);
         let false_expr = Self::from_expression_node(false_expr_n.clone(), ctx);
         let result_ty = Self::common_target_type_for_type_list(
             [true_expr.ty(), false_expr.ty()].iter().cloned(),
         );
-        let true_expr = true_expr.maybe_convert_to(result_ty.clone(), &true_expr_n, &mut ctx.diag);
-        let false_expr = false_expr.maybe_convert_to(result_ty, &false_expr_n, &mut ctx.diag);
+        let true_expr = true_expr.maybe_convert_to(result_ty.clone(), &true_expr_n, ctx.diag);
+        let false_expr = false_expr.maybe_convert_to(result_ty, &false_expr_n, ctx.diag);
         Expression::Condition {
             condition: Box::new(condition),
             true_expr: Box::new(true_expr),
@@ -827,6 +853,25 @@ impl Expression {
         }
     }
 
+    fn from_index_expression_node(
+        node: syntax_nodes::IndexExpression,
+        ctx: &mut LookupCtx,
+    ) -> Expression {
+        let (array_expr_n, index_expr_n) = node.Expression();
+        let array_expr = Self::from_expression_node(array_expr_n, ctx);
+        let index_expr = Self::from_expression_node(index_expr_n.clone(), ctx).maybe_convert_to(
+            Type::Int32,
+            &index_expr_n,
+            &mut ctx.diag,
+        );
+
+        let ty = array_expr.ty();
+        if !matches!(ty, Type::Array(_) | Type::Invalid) {
+            ctx.diag.push_error(format!("{} is not an indexable type", ty), &node);
+        }
+        Expression::ArrayIndex { array: Box::new(array_expr), index: Box::new(index_expr) }
+    }
+
     fn from_object_literal_node(
         node: syntax_nodes::ObjectLiteral,
         ctx: &mut LookupCtx,
@@ -1031,10 +1076,10 @@ fn maybe_lookup_object(
         let next_str = crate::parser::normalize_identifier(next.text());
         ctx.current_token = Some(next.clone().into());
         match base.lookup(ctx, &next_str) {
-            Some(result) => {
-                base = result.expression;
+            Some(LookupResult::Expression { expression, .. }) => {
+                base = expression;
             }
-            None => {
+            _ => {
                 if let Some(minus_pos) = next.text().find('-') {
                     if base.lookup(ctx, &next.text()[0..minus_pos]).is_some() {
                         ctx.diag.push_error(format!("Cannot access the field '{}'. Use space before the '-' if you meant a subtraction", next.text()), &next);
