@@ -1,12 +1,6 @@
-/* LICENSE BEGIN
-    This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
+// Copyright © SixtyFPS GmbH <info@sixtyfps.io>
+// SPDX-License-Identifier: (GPL-3.0-only OR LicenseRef-SixtyFPS-commercial)
 
-    SPDX-License-Identifier: GPL-3.0-only
-    This file is also available under commercial licensing terms.
-    Please contact info@sixtyfps.io for more information.
-LICENSE END */
 use std::borrow::Cow;
 use std::cell::RefCell;
 use std::collections::{BTreeMap, HashMap, HashSet};
@@ -106,7 +100,7 @@ impl<'a> TypeLoader<'a> {
     ) -> Vec<ImportedTypes> {
         let dependencies = self.collect_dependencies(doc, diagnostics).await;
         let mut foreign_imports = vec![];
-        for mut import in dependencies.into_iter() {
+        for mut import in dependencies {
             if import.file.ends_with(".60") {
                 if let Some(imported_types) =
                     ImportedName::extract_imported_names(&import.imported_types)
